/* vim: set shiftwidth=2 tabstop=8 autoindent cindent expandtab: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

/**
 * Transition events are defined in:
 * http://www.w3.org/TR/css3-transitions/#transition-events-
 * http://dev.w3.org/csswg/css3-transitions/#transition-events-
 */

[scriptable, builtinclass, uuid(9013310a-e376-40bc-b141-9b9ae3085daa)]
interface nsIDOMTransitionEvent : nsIDOMEvent {
  readonly attribute DOMString           propertyName;
  readonly attribute float               elapsedTime;
  void               initTransitionEvent(in DOMString typeArg, 
                                         in boolean canBubbleArg, 
                                         in boolean cancelableArg, 
                                         in DOMString propertyNameArg,
                                         in float elapsedTimeArg);
};
